/*
 * jGuild Project: jRPM
 * Released under the Apache License ( http://www.apache.org/LICENSE )
 */
package com.jguild.jrpm.io;

import java.io.DataInputStream;
import java.io.IOException;
import java.util.Comparator;
import java.util.HashMap;
import java.util.TreeSet;

import java.util.logging.Logger;
import java.util.logging.Level;

import com.jguild.jrpm.io.datatype.DataTypeIf;
import com.jguild.jrpm.io.datatype.TypeFactory;

/**
 * This class represents the abstract definition of a header structur.
 * It can be either a signature or a header. The tags of such a structure
 * can be accessed by either their tag id or by their tag name.
 * Also all available and all read tag names in this structure can be accessed.
 *
 * @author kuss
 * @version $Id: Header.java,v 1.10 2004/09/09 09:52:33 pnasrat Exp $
 */
public abstract class Header {
    private static final int HEADER_LENGTH = 16;
    private static final Logger logger = RPMFile.logger;
    private HashMap store = new HashMap();
    private IndexEntry[] indexes;
    private int version;
    private long indexDataSize;
    private long indexNumber;
    private boolean rawHeader = false;
    /**
     * The size in bytes of this structure.
     */
    protected long size;

    /**
     * Create a header structure from an input stream.
     * <p/>
     * The header structure of a signature or a header can be read and
     * also the index entries containing the tags for this rpm section
     * (signature or header).
     * <p/>
     * Unless we have a raw header from headerUnload or the database,
     * a header is read consisting of the following fields:
     * <code><pre>
     * byte magic[3];      (3  byte)  (8e ad e8)
     * int version;        (1  byte)
     * byte reserved[4];   (4  byte)
     * long num_index;     (4  byte)
     * long num_data;      (4  byte)
     * </pre></code>
     * <p/>
     * Afterwards the index entries are read and then the tags and the
     * correspondig data entries are read.
     *
     * @param inputStream An inputstream containing rpm file informations
     * @param rawHeader   Are we a raw header (from headerUnload or rpmdb)
     * @throws IOException if an error occurs on reading informations
     *                     out of the stream
     */
    public Header(DataInputStream inputStream, boolean rawHeader) throws IOException {
        if (logger.isLoggable(Level.FINER)) {
            logger.finer("Start Reading Header");
        }

        if (!rawHeader) {
            // Read header
            size = HEADER_LENGTH;

            check(inputStream.readUnsignedByte() == 0x8E);
            check(inputStream.readUnsignedByte() == 0xAD);
            check(inputStream.readUnsignedByte() == 0xE8);
            version = inputStream.readUnsignedByte();

            if (logger.isLoggable(Level.FINER)) {
                logger.finer("version: " + version);
            }

            // skip reserved bytes
            inputStream.skipBytes(4);
        }

        indexNumber = inputStream.readInt();

        if (logger.isLoggable(Level.FINER)) {
            logger.finer("indexes available: " + indexNumber);
        }

        indexDataSize = inputStream.readInt();

        if (logger.isLoggable(Level.FINER)) {
            logger.finer("index data size: " + indexDataSize);
        }

        // Read indexes
        // make sure to sort them in order of offset to
        // be able to read the store without jumping arround in
        // the file
        TreeSet _indexes = new TreeSet(new Comparator() {
            public int compare(Object o1, Object o2) {
                return (int) (((IndexEntry) o1).getOffset() - ((IndexEntry) o2).getOffset());
            }

            public boolean equals(Object o) {
                return false;
            }
        });

        for (int i = 0; i < indexNumber; i++) {
            IndexEntry index = new IndexEntry(inputStream);

            _indexes.add(index);
            size += index.getSize();
        }

        indexes = new IndexEntry[0];
        indexes = (IndexEntry[]) _indexes.toArray(indexes);

        // Read store
        for (int i = 0; i < indexes.length; i++) {
            IndexEntry index = indexes[i];

            //            if (index.getType().equals(RPMIndexType.STRING_ARRAY) || index.getType().equals(RPMIndexType.STRING) ||
            //                    index.getType().equals(RPMIndexType.I18NSTRING)) {
            //                if (i < (indexes.length - 1)) {
            //                    IndexEntry next = indexes[i + 1];
            //
            //                    length = next.getOffset() - index.getOffset();
            //                } else {
            //                    length = indexDataSize - index.getOffset();
            //                }
            //
            //                // and initialize temporary space for data
            //                stringData = new byte[(int) length];
            //
            //                // and read it from stream
            //                inputStream.readFully(stringData);
            //            }
            DataTypeIf dataObject = null;

            if (logger.isLoggable(Level.FINER)) {
                logger.finer("Reading for tag '" + getTagNameForId(index.getTag()) + "' '" + index.getCount() + "' entries of type '" +
                        index.getType().getName() + "'");
            }

            dataObject = TypeFactory.createFromStream(inputStream, index,
                    (i < (indexes.length - 1)) ? (indexes[i + 1].getOffset() - index.getOffset()) : (indexDataSize - index.getOffset()));

            // adjust size
            size += dataObject.getSize();

            store.put(new Long(index.getTag()), dataObject);
        }

        if (logger.isLoggable(Level.FINER)) {
            logger.finer("");
        }

        if (logger.isLoggable(Level.FINER)) {
            logger.finer("Finished Reading Header");
        }
    }

    /**
     * Construct a header structure for the given input stream.
     *
     * @param inputStream
     * @throws IOException
     */
    public Header(DataInputStream inputStream) throws IOException {
        this(inputStream, false);
    }

    /**
     * Read all known tag names for this header structure.
     *
     * @return An array of tag names
     */
    public static String[] getKnownTagNames() {
        return new String[0];
    }

    /**
     * Get the size in bytes of this structure
     *
     * @return The size in bytes.
     */
    public long getSize() {
        return size;
    }

    /**
     * Get a tag by id as a Long
     *
     * @param tag A tag id as a Long
     * @return A data struct containing the data of this tag
     */
    public DataTypeIf getTag(Long tag) {
        return (DataTypeIf) store.get(tag);
    }

    /**
     * Get a tag by id as a long
     *
     * @param tag A tag id as a long
     * @return A data struct containing the data of this tag
     */
    public DataTypeIf getTag(long tag) {
        return getTag(new Long(tag));
    }

    /**
     * Get a tag by name
     *
     * @param tagname A tag name
     * @return A data struct containing the data of this tag
     */
    public DataTypeIf getTag(String tagname) {
        return getTag(getTagIdForName(tagname));
    }

    /**
     * Set a tag by id as a Long
     *
     * @param tag  A tag id as a Long
     * @param data A data struct containing the data of this tag
     */
    public void setTag(Long tag, DataTypeIf data) {
        isValidTag(tag.longValue());
        store.put(tag, data);
    }

    /**
     * Set a tag by id as a long
     *
     * @param tag  A tag id as a long
     * @param data A data struct containing the data of this tag
     */
    public void setTag(long tag, DataTypeIf data) {
        setTag(new Long(tag), data);
    }

    /**
     * Set a tag by id as a string
     *
     * @param tagname A tag id as a string
     * @param data    A data struct containing the data of this tag
     */
    public void setTag(String tagname, DataTypeIf data) {
        setTag(getTagIdForName(tagname), data);
    }

    /**
     * Get all tag ids contained in this rpm file.
     *
     * @return All tag ids contained in this rpm file.
     */
    public long[] getTagIds() {
        Long[] tmp = (Long[]) store.keySet().toArray(new Long[0]);
        long[] ret = new long[tmp.length];

        for (int i = 0; i < tmp.length; i++) {
            ret[i] = tmp[i].longValue();
        }

        return ret;
    }

    /**
     * Get all tag names contained in this rpm file.
     *
     * @return All tag names contained in this rpm file.
     */
    public String[] getTagNames() {
        Long[] tmp = (Long[]) store.keySet().toArray(new Long[0]);
        String[] ret = new String[tmp.length];

        for (int i = 0; i < tmp.length; i++) {
            ret[i] = getTagNameForId(tmp[i].longValue());
        }

        return ret;
    }

    /**
     * Asserts a boolean value and throws an exception if it
     * is false
     *
     * @param test A boolean test variable
     * @throws IOException if the variable test is false
     */
    private static final void check(boolean test)
            throws IOException {
        if (!test) {
            throw new IOException("Corrupted archive");
        }
    }

    /**
     * Read a tag with a given tag name. The tag will be read out of the
     * class defined in getTagEnum().
     *
     * @param tagname A RPM tag name
     * @return The id of the RPM tag
     * @throws IllegalArgumentException if the tag name was not found
     */
    public abstract long getTagIdForName(String tagname);

    /**
     * Read a tag with a given tag id. The tag will be read out of the
     * class defined in getTagEnum().
     *
     * @param tagid A RPM tag id
     * @return The name of the RPM tag
     * @throws IllegalArgumentException if the tag id was not found
     */
    public abstract String getTagNameForId(long tagid);

    /**
     * Test if the given tagid is associated with a valid tag
     *
     * @param tagid The id of a tag
     * @return TRUE if the tagid is valid
     */
    public abstract boolean isValidTag(long tagid);

    /**
     * Test if the given tagname is associated with a valid tag
     *
     * @param tagname The name of a tag
     * @return TRUE if the tagname is valid
     */
    public abstract boolean isValidTag(String tagname);
}
