/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/
package com.jguild.jrpm.io;

import com.jguild.jrpm.io.bzip2.CBZip2InputStream;
import com.jguild.jrpm.io.cpio.CPIOEntry;
import com.jguild.jrpm.io.cpio.CPIOInputStream;
import com.jguild.jrpm.io.datatype.DataTypeIf;
import com.jguild.jrpm.io.datatype.I18NSTRING;
import com.jguild.jrpm.io.datatype.STRING_ARRAY;
import com.jguild.jrpm.io.datatype.TypeFactory;

import java.io.*;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.GZIPInputStream;

/**
 * This class allows IO access to an RPM file.
 *
 * @todo Implement equals()
 */
public class RPMFile {
    public static final Logger logger = Logger.getLogger("jrpm.io");
    private RPMHeader header = null;
    private RPMLead lead = null;
    private RPMSignature signature = null;
    private int localePosition;
    private File rpmFile = null;
    private boolean editingRpmFile = false;

    /**
     * Creates a new empty RPMFile object.
     */
    public RPMFile() {
    }

    /**
     * Creates a new RPMFile object out of a file.
     *
     * @param fh The file object representing a rpm file
     */
    public RPMFile(File fh) {
        rpmFile = fh;
    }

    private synchronized void reset() {
        header = null;
        lead = null;
        signature = null;
        editingRpmFile = false;
    }

    /**
     * Set the file this RPMFile should represent
     *
     * @param fh The file object representing a rpm file
     */
    public synchronized void setFile(File fh) {
        if (editingRpmFile) {
            throw new IllegalStateException("RPM file is currently edited");
        }
        rpmFile = fh;
        reset();
    }

    /**
     * Parse the RPMFile and will extract all informations. This must be called
     * before any informations can be read from the rpm file.
     *
     * @throws IOException If an error occurs during read of the rpm file
     */
    public synchronized void parse() throws IOException {
        if (rpmFile == null)
            throw new IllegalStateException("A file must be specified");

        if (!rpmFile.exists())
            throw new IllegalStateException(
                    "The specified file does not exist");

        try {
            readFromStream(new BufferedInputStream(new FileInputStream(rpmFile)));
        } catch (IOException e) {
            reset();
            throw e;
        }
        editingRpmFile = true;
    }

    /**
     * Get the header section of this rpm file.
     *
     * @return The rpm header
     */
    public synchronized RPMHeader getHeader() {
        if (header == null)
            throw new IllegalStateException(
                    "There are no header informations");

        return header;
    }

    /**
     * Get all known tags of this rpm file. This is equivalent to the
     * --querytags option in rpm.
     *
     * @return An array of all tag names
     */
    public static String[] getKnownTagNames() {
        return Header.getKnownTagNames();
    }

    /**
     * Get the lead section of this rpm file
     *
     * @return The rpm lead
     */
    public synchronized RPMLead getLead() {
        if (lead == null)
            throw new IllegalStateException(
                    "There are no lead informations");

        return lead;
    }

    /**
     * Set the locale as int for all I18N strings that are returned by
     * getTag(). The position has to correspond with the same position in the
     * array returned by getLocales().
     *
     * @param pos The position in the array returned by getLocales().
     */
    public synchronized void setLocale(int pos) {
        localePosition = pos;
    }

    /**
     * Set the locale as string for all I18N strings that are returned by
     * getTag(). The string must match with a string returned by getLocales().
     *
     * @param locale A locale matching a locale returned by getLocales()
     * @throws IllegalArgumentException If the locale is not defined by getLocales().
     */
    public synchronized void setLocale(String locale) {
        String[] locales = ((STRING_ARRAY) getTag("HEADERI18NTABLE")).getData();

        for (int pos = 0; pos < locales.length; pos++) {
            if (locales[pos].equals(locale)) {
                setLocale(pos);

                return;
            }
        }

        throw new IllegalArgumentException("Unknown locale <" + locale + ">");
    }

    /**
     * Return all known locales that are supported by this RPM file. The array
     * is read out of the RPM file with the tag "HEADERI18NTABLE". The RPM has
     * one entry for all I18N strings defined by this tag.
     *
     * @return A string array of all defined locales
     */
    public synchronized String[] getLocales() {
        return ((STRING_ARRAY) getTag("HEADERI18NTABLE")).getData();
    }

    /**
     * Get the signature section of this rpm file
     *
     * @return The rpm signature
     */
    public synchronized RPMSignature getSignature() {
        if (signature == null)
            throw new IllegalStateException(
                    "There are no signature informations");

        return signature;
    }

    /**
     * Get a tag by id as a Long
     *
     * @param tag A tag id as a Long
     * @return A data struct containing the data of this tag
     */
    public synchronized DataTypeIf getTag(Long tag) {
        DataTypeIf data = getHeader().getTag(tag);

        // set the locale for all I18N strings
        if (data instanceof I18NSTRING) {
            ((I18NSTRING) data).setLocaleIndex(localePosition);
        }

        return data;
    }

    /**
     * Get a tag by id as a long
     *
     * @param tag A tag id as a long
     * @return A data struct containing the data of this tag
     */
    public synchronized DataTypeIf getTag(long tag) {
        return getTag(new Long(tag));
    }

    /**
     * Get a tag by name
     *
     * @param tagname A tag name
     * @return A data struct containing the data of this tag
     */
    public synchronized DataTypeIf getTag(String tagname) {
        return getTag(getTagIdForName(tagname));
    }

    /**
     * Read a tag with a given tag name.
     *
     * @param tagname A RPM tag name
     * @return The id of the RPM tag
     * @throws IllegalArgumentException if the tag name was not found
     * @see Header#getTagIdForName(String)
     */
    public synchronized long getTagIdForName(String tagname) {
        return getHeader().getTagIdForName(tagname);
    }

    /**
     * Get all tag ids contained in this rpm file.
     *
     * @return All tag ids contained in this rpm file.
     */
    public synchronized long[] getTagIds() {
        return getHeader().getTagIds();
    }

    /**
     * Read a tag with a given tag id.
     *
     * @param tagid A RPM tag id
     * @return The name of the RPM tag
     * @throws IllegalArgumentException if the tag id was not found
     * @see Header#getTagNameForId(long)
     */
    public synchronized String getTagNameForId(long tagid) {
        return getHeader().getTagNameForId(tagid);
    }

    /**
     * Get all tag names contained in this rpm file.
     *
     * @return All tag names contained in this rpm file.
     */
    public synchronized String[] getTagNames() {
        return getHeader().getTagNames();
    }

    /**
     * Read informations of a rpm file out of an input stream.
     *
     * @param rpmInputStream The input stream representing the rpm file
     * @throws IOException if an error occurs during read of the rpm file
     */
    private void readFromStream(InputStream rpmInputStream) throws IOException {
        InputStream inputStream = new DataInputStream(rpmInputStream);

        lead = new RPMLead((DataInputStream) inputStream);
        signature = new RPMSignature((DataInputStream) inputStream);

        if (logger.isLoggable(Level.FINER)) {
            logger.finer("Signature Size: " + signature.getSize());
        }

        header = new RPMHeader((DataInputStream) inputStream);

        if (logger.isLoggable(Level.FINER)) {
            logger.finer("Header Size: " + header.getSize());
        }

        final DataTypeIf payloadTag = getTag("PAYLOADFORMAT");
        final String payloadFormat = payloadTag != null ?
                payloadTag.toString() : "cpio";
        final DataTypeIf payloadCompressionTag = getTag("PAYLOADCOMPRESSOR");
        final String payloadCompressor = payloadCompressionTag != null ?
                payloadCompressionTag.toString() : "gzip";

        if (payloadFormat.equals("cpio")) {
            if (logger.isLoggable(Level.FINER)) {
                logger.finer("PAYLOADCOMPRESSOR: " + payloadCompressor);
            }

            if (payloadCompressor.equals("gzip")) {
                inputStream = new GZIPInputStream(rpmInputStream);
            } else if (payloadCompressor.equals("bzip2")) {
                inputStream = new CBZip2InputStream(rpmInputStream);
            } else if (payloadCompressor.equals("none")) {
                inputStream = rpmInputStream;
            } else {
                throw new IOException("Unsupported compressor type "
                        + payloadCompressor);
            }

            ByteCountInputStream countInputStream = new ByteCountInputStream(
                    inputStream);
            CPIOInputStream cpioInputStream = new CPIOInputStream(
                    countInputStream);
            CPIOEntry readEntry;
            ArrayList fileNamesList = new ArrayList();
            String fileEntry;
            while ((readEntry = cpioInputStream.getNextEntry()) != null) {
                if (logger.isLoggable(Level.FINER)) {
                    logger.finer("Read CPIO entry: " + readEntry.getName()
                            + " ;mode:" + readEntry.getMode());
                }
                if (readEntry.isRegularFile() || readEntry.isSymbolicLink()
                        || readEntry.isDirectory()) {
                    fileEntry = readEntry.getName();
                    if (fileEntry.startsWith("./"))
                        fileEntry = fileEntry.substring(1);
                    fileNamesList.add(fileEntry);
                }
            }
            getHeader().setTag(
                    "FILENAMES",
                    TypeFactory.createSTRING_ARRAY((String[]) fileNamesList
                            .toArray(new String[0])));
        } else {
            throw new IOException("Unsupported Payload type " + payloadFormat);
        }
        // TODO check ARCHIVESIZE with countInputStream.getCount();

        // filling in signatures
        // TODO: check signatures!
        setHeaderTagFromSignature("SIGSIZE", "SIZE");
        setHeaderTagFromSignature("SIGLEMD5_1", "LEMD5_1");
        setHeaderTagFromSignature("SIGPGP", "PGP");
        setHeaderTagFromSignature("SIGLEMD5_2", "LEMD5_2");
        setHeaderTagFromSignature("SIGMD5", "MD5");
        setHeaderTagFromSignature("SIGGPG", "GPG");
        setHeaderTagFromSignature("SIGPGP5", "PGP5");
        setHeaderTagFromSignature("BADSHA1_1", "BADSHA1_1");
        setHeaderTagFromSignature("BADSHA1_2", "BADSHA1_2");
        setHeaderTagFromSignature("DSAHEADER", "DSA");
        setHeaderTagFromSignature("RSAHEADER", "RSA");
        setHeaderTagFromSignature("SHA1HEADER", "SHA1");
        setHeaderTagFromSignature("ARCHIVESIZE", "PAYLOADSIZE");
        rpmInputStream.close();

    }

    private void setHeaderTagFromSignature(String headerTag, String signatureTag) {
        if (getHeader().getTag(headerTag) == null)
            getHeader().setTag(headerTag,
                    getSignature().getTag(signatureTag));
    }

    /**
     * Release locked resources.
     */
    public void close() {
        reset();
    }

    /**
     * Same as doing toXML(true).
     *
     * @return String containing the XML representation of this RPM.
     * @see #toXML(boolean)
     */
    public String toXML() {
        StringWriter buf = new StringWriter();
        try {
            toXML(buf, true);
            buf.flush();
            return buf.toString();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * Returns an XML version of this file
     *
     * @param excludePayload If this is true, the payload will not be included in the XML.
     * @return XML rpm.
     */
    public String toXML(boolean excludePayload) {
        StringWriter buf = new StringWriter();
        try {
            toXML(buf, excludePayload);
            buf.flush();
            return buf.toString();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * Outputs this rpm in an XML format to the specified i/o writer.
     *
     * @param writer         Writer stream.
     * @param excludePayload If this is true, the payload will not be included in the XML.
     * @throws IOException If an error occurred writing to the writer.
     */
    public void toXML(Writer writer, boolean excludePayload) throws IOException {
        // TODO
    }

    private class ByteCountInputStream extends FilterInputStream {
        private int count = 0;

        public ByteCountInputStream(InputStream is) {
            super(is);
        }

        public int getCount() {
            return count;
        }

        public int read() throws IOException {
            count++;
            return in.read();
        }

        public int read(byte b[]) throws IOException {
            int size = read(b, 0, b.length);
            count += size;
            return size;
        }

        public int read(byte b[], int off, int len) throws IOException {
            int size = in.read(b, off, len);
            count += size;
            return size;
        }

        public long skip(long n) throws IOException {
            long size = in.skip(n);
            count += size;
            return size;
        }
    }

    /**
     * Load an RPM file using the native rpm executables.
     *
     * @param file RPM file.
     */
    public static RPMFile loadUsingNative(File file) {
        return null; // TODO
    }
}
