/*
 * jGuild Project: jRPM
 * Released under the Apache License ( http://www.apache.org/LICENSE )
 */
package com.jguild.jrpm.io;

import java.io.DataInputStream;
import java.io.IOException;

import java.util.logging.Logger;
import java.util.logging.Level;

import com.jguild.jrpm.io.constant.EnumIf;
import com.jguild.jrpm.io.constant.RPMSignatureTag;

/**
 * RPM Signature.
 */
public class RPMSignature extends Header {
    private static final Logger logger = RPMFile.logger;

    /**
     * Creates a new RPMSignature object from an input stream
     *
     * @param inputStream The input stream
     * @throws IOException if an error occurs on reading informations out of the stream
     */
    public RPMSignature(DataInputStream inputStream) throws IOException {
        super(inputStream);

        // Make signature size modulo 8 = 0
        long fill = (size % 8L);

        if (fill != 0) {
            fill = 8 - fill;
        }

        if (logger.isLoggable(Level.FINER)) {
            logger.finer("skip " + fill + " bytes for signature");
        }

        size += inputStream.skip(fill);
    }

    /**
     * @see com.jguild.jrpm.io.Header#getKnownTagNames()
     */
    public static String[] getKnownTagNames() {
        return RPMSignatureTag.getEnumNames();
    }

    /**
     * @see com.jguild.jrpm.io.Header#getTagIdForName(java.lang.String)
     */
    public long getTagIdForName(String tagname) {
        EnumIf e = RPMSignatureTag.getEnumByName(tagname);
        if (e == null) {
            throw new IllegalArgumentException("unknown tag with name <" + tagname + ">");
        }
        return e.getId();
    }

    /**
     * @see com.jguild.jrpm.io.Header#getTagNameForId(long)
     */
    public String getTagNameForId(long tagid) {
        EnumIf e = RPMSignatureTag.getEnumById(tagid);
        if (e == null) {
            throw new IllegalArgumentException("unknown tag with id <" + tagid + ">");
        }
        return e.getName();
    }

    /**
     * @see com.jguild.jrpm.io.Header#isValidTag(long)
     */
    public boolean isValidTag(long tagid) {
        return RPMSignatureTag.getEnumById(tagid) != null;
    }

    /**
     * @see com.jguild.jrpm.io.Header#isValidTag(java.lang.String)
     */
    public boolean isValidTag(String tagname) {
        return RPMSignatureTag.getEnumByName(tagname) != null;
    }
}
